package de.sh_edraft.data

import de.sh_edraft.config.Config
import java.net.URI
import java.net.http.HttpClient
import java.net.http.HttpRequest
import java.net.http.HttpResponse
import java.util.logging.Logger
import org.json.JSONObject

class DataService(
    private val logger: Logger,
    private val config: Config
) {
    private fun getUserIdByPlayerIdQuery(id: String): String {
        return JSONObject(
            """
            {
                "query": "query { users(filter: { minecraftId: \"$id\" }) { id } }"
            }
        """.trimIndent()
        ).toString()
    }

    private fun getUserJoinedMutation(id: Int): String {
        return JSONObject(
            """
            {
                "query": "mutation { userJoinedGameServer { userJoined(input: { userId: $id gameServer: \"Minecraft\" }) { id } } }"
            }
        """.trimIndent()
        ).toString()
    }

    private fun getUserLeavedMutation(id: Int): String {
        return JSONObject(
            """
            {
                "query": "mutation { userJoinedGameServer { userLeaved (input: { userId: $id gameServer: \"Minecraft\" }) { id } } }"
            }
        """.trimIndent()
        ).toString()
    }

    fun getPlayerGlobalId(name: String): String? {
        val client = HttpClient.newBuilder().build();
        val request = HttpRequest.newBuilder()
            .uri(URI.create("https://api.mojang.com/users/profiles/minecraft/$name"))
            .build();

        val response = client.send(request, HttpResponse.BodyHandlers.ofString()).body() ?: return null;
        val json = JSONObject(response)
        return json.getString("id");
    }


    fun getUserIdByPlayerId(id: String): Int? {
        val client = HttpClient.newBuilder().build();

        val request = HttpRequest.newBuilder()
            .uri(URI.create(this.config.ApiURL))
            .POST(HttpRequest.BodyPublishers.ofString(this.getUserIdByPlayerIdQuery(id)))
            .header("Authorization", "API-Key ${this.config.ApiKey}")
            .header("Content-Type", "application/json")
            .build();

        val response = client.send(request, HttpResponse.BodyHandlers.ofString());
        try {
            return JSONObject(response.body())
                .getJSONObject("data")
                .getJSONArray("users")
                .getJSONObject(0)
                .getString("id")
                .toIntOrNull();
        } catch (e: Exception) {
            logger.severe(e.message);
        }
        return null;
    }

    fun sendJoinedPlayer(id: Int) {
        val client = HttpClient.newBuilder().build();

        val request = HttpRequest.newBuilder()
            .uri(URI.create(this.config.ApiURL))
            .POST(HttpRequest.BodyPublishers.ofString(this.getUserJoinedMutation(id)))
            .header("Authorization", "API-Key ${this.config.ApiKey}")
            .header("Content-Type", "application/json")
            .build();

        val response = client.send(request, HttpResponse.BodyHandlers.ofString());
        try {
            // check if response is valid
            val joinId = JSONObject(response.body())
                .getJSONObject("data")
                .getJSONObject("userJoinedGameServer")
                .getJSONObject("userJoined")
                .getString("id")
                .toIntOrNull();
        } catch (e: Exception) {
            logger.severe(e.message);
        }
    }

    fun sendLeavedPlayer(id: Int) {
        val client = HttpClient.newBuilder().build();

        val request = HttpRequest.newBuilder()
            .uri(URI.create(this.config.ApiURL))
            .POST(HttpRequest.BodyPublishers.ofString(this.getUserLeavedMutation(id)))
            .header("Authorization", "API-Key ${this.config.ApiKey}")
            .header("Content-Type", "application/json")
            .build();

        val response = client.send(request, HttpResponse.BodyHandlers.ofString());
        try {
            // check if response is valid
            val joinId = JSONObject(response.body())
                .getJSONObject("data")
                .getJSONObject("userJoinedGameServer")
                .getJSONObject("userJoined")
                .getString("id")
                .toIntOrNull();
        } catch (e: Exception) {
            logger.severe(e.message);
        }
    }
}